﻿using System.Collections;

using anbs_cp.Structs;

namespace anbs_cp.Classes;

/// <summary>
/// Список из пара ключ-значение
/// </summary>
/// <typeparam name="TK">Тип ключа</typeparam>
/// <typeparam name="TV">Тип значения</typeparam>
public class KeyValueList<TK, TV>: IEnumerable<KeyValue<TK, TV>>
{
    /// <summary>
    /// Хранение значений
    /// </summary>
    private readonly List<KeyValue<TK, TV>> _list;

    #region Конструкторы
    /// <summary>
    /// Конструктор по умолчанию
    /// </summary>
    public KeyValueList () => _list = new();
    #endregion

    #region Свойства
    /// <summary>
    /// Список ключей из списка
    /// </summary>
    public IEnumerable<TK> Keys => GetKeys();
    #endregion

    #region Методы

    /// <summary>
    /// Получает список ключей
    /// </summary>
    /// <returns>Список ключей</returns>
    private IEnumerable<TK> GetKeys () => from keyValue in _list where keyValue.Key is not null select keyValue.Key;

    /// <summary>
    /// Добавляет в список параметр
    /// </summary>
    /// <param name="keyValue">Параметр</param>
    public void Add (KeyValue<TK, TV> keyValue) => _list.Add(keyValue);

    /// <summary>
    /// Добавляет в список параметр
    /// </summary>
    /// <param name="key">Ключ параметра</param>
    /// <param name="value">Значение</param>
    public void Add (TK key, TV value) => _list.Add(new(key, value));

    /// <summary>
    /// Изменяет значение
    /// </summary>
    /// <param name="keyValue">Новое значение</param>
    public void ChangeValue (KeyValue<TK, TV> keyValue)
    {
        // Если такой ключ не существует
        if (!Contains(keyValue.Key!))
        {
            // - тогда добавляю новое значение
            Add(keyValue);

            // - прерываю
            return;
        }

        // Существующее значение
        KeyValue<TK, TV> existValue = GetItem(keyValue.Key!) ?? new();

        // Удаляем существующее
        _list.Remove(existValue);

        // Добавляем новое
        _list.Add(keyValue);
    }

    /// <summary>
    /// Изменяет значение
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <param name="newValue">Новое значение</param>
    public void ChangeValue (TK key, TV newValue) => ChangeValue(new(key, newValue));

    /// <summary>
    /// Получает элемент по ключу
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <returns>Элемент <see cref="KeyValue{TK,TV}"/></returns>
    public KeyValue<TK, TV>? GetItem (TK key)
    {
        // Ищем элемент в списке
        foreach (KeyValue<TK, TV> keyValueItem in _list.Where(keyValueItem => keyValueItem.Key!.Equals(key)))
            // - возвращаем его при нахождении
            return keyValueItem;

        // Элемент не найден -- вывожу null
        return null;
    }

    /// <summary>
    /// Получает значение
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <returns>Значение</returns>
    public TV? GetValue (TK key)
    {
        // Если такой ключ не существует
        if (!Contains(key))
            // Тогда возвращаю значение по умолчанию
            return default;

        // Получаю элемент
        KeyValue<TK, TV> keyValue = GetItem(key) ?? new();

        // Вывожу значение
        return keyValue.Value;
    }

    /// <summary>
    /// Удаляет элемент из списка
    /// </summary>
    /// <param name="keyValue">Элемент</param>
    public void Remove (KeyValue<TK, TV> keyValue) => _list.Remove(keyValue);

    /// <summary>
    /// Удаляет элемент из списка
    /// </summary>
    /// <param name="key">Ключ элемента</param>
    public void Remove (TK key) => Remove(GetItem(key) ?? new());

    /// <summary>
    /// Проверяет, содержится ли элемент в списке
    /// </summary>
    /// <param name="keyValue">Элемент</param>
    /// <returns>Результат проверки</returns>
    public bool Contains (KeyValue<TK, TV> keyValue) => _list.Contains(keyValue);

    /// <summary>
    /// Проверяет, содержится ли элемент в списке
    /// </summary>
    /// <param name="key">Ключ элемента</param>
    /// <returns>Результат проверки</returns>
    public bool Contains (TK key) => Keys.Any(keyParam => keyParam!.Equals(key));

    #endregion

    #region Реализация интерфейса IEnumerable<KeyValue<TK, TV>>
    /// <summary>
    /// Получаю <see cref="IEnumerator"/>
    /// </summary>
    /// <returns><see cref="IEnumerator"/></returns>
    public IEnumerator<KeyValue<TK, TV>> GetEnumerator () => _list.GetEnumerator();

    /// <summary>
    /// Получаю <see cref="IEnumerator"/>
    /// </summary>
    /// <returns><see cref="IEnumerator"/></returns>
    IEnumerator IEnumerable.GetEnumerator () => GetEnumerator();
    #endregion
}