﻿using System.Net.Mail;
using System.Text.RegularExpressions;

namespace anbs_cp.Classes;

/// <summary>
/// Форматирование текста
/// </summary>
public static class TextFormatter
{
    /// <summary>
    /// Заменяет %МАРКЕРЫ% на их значения
    /// </summary>
    /// <param name="message">Текст сообщения</param>
    /// <param name="replaceDictionary">Словарь замен</param>
    /// <returns>Отформатированное сообщение</returns>
    public static string FormatMessage (string message, Dictionary<string, string> replaceDictionary) =>
        replaceDictionary.Aggregate(message,
            static (current, item) => current.Replace($"%{item.Key}%", item.Value));

    /// <summary>
    /// Обрезает строку до указанных в параметре <paramref name="maxLength"/> символов
    /// </summary>
    /// <param name="text">Текст, который нужно обрезать</param>
    /// <param name="maxLength">Максимальное количество символов в тексте</param>
    /// <param name="endDots">Чем завершать обрезанный текст, если он был обрезан. Внимание расходует <see cref="maxLength"/>!</param>
    /// <returns>Обрезанный текст</returns>
    public static string GetShortText (string text, int maxLength, string endDots = "") =>
        text.Length < maxLength ? text : $"{text[..(maxLength - endDots.Length)]}{endDots}";

    /// <summary>
    /// Генерирует случайный пароль, удовлетворяющий параметрам <see cref="PasswordOptions"/> <paramref name="options"/>.
    /// Автор метода: Darkseal (https://stackoverflow.com/users/1233379/darkseal)
    /// URL: https://stackoverflow.com/a/46229180/16469671
    /// </summary>
    /// <param name="options">Объект допустимых параметров пароля, содержащий требования к надежности пароля.</param>
    /// <returns>Случайный пароль</returns>
    public static string GenerateRandomPassword (PasswordOptions? options)
    {
        //Проверка options и установка по-умолчанию
        options ??= new();

        //Получаю массив символов
        string[] randomChars = {
            "ABCDEFGHJKLMNOPQRSTUVWXYZ",    // прописные буквы
            "abcdefghijkmnopqrstuvwxyz",    // строчные буквы
            "0123456789",                   // цифры
            "~!@#$%^&*+-/.,{}[]();:|?<>='`" // символы
        };

        //Создаю объект Random
        Random rand = new(Environment.TickCount);

        //Массив результатов
        List<char> chars = new();

        //Вставляю прописные буквы
        if (options.RequireUppercase)
            chars.Insert(rand.Next(0, chars.Count),
                randomChars[0][rand.Next(0, randomChars[0].Length)]);

        //Вставляю строчные буквы
        if (options.RequireLowercase)
            chars.Insert(rand.Next(0, chars.Count),
                randomChars[1][rand.Next(0, randomChars[1].Length)]);

        //Вставляю цифры
        if (options.RequireDigit)
            chars.Insert(rand.Next(0, chars.Count),
                randomChars[2][rand.Next(0, randomChars[2].Length)]);

        //Вставляю символы
        if (options.RequireNonAlphanumeric)
            chars.Insert(rand.Next(0, chars.Count),
                randomChars[3][rand.Next(0, randomChars[3].Length)]);

        //Делаю выборку
        for (int i = chars.Count; i < options.RequiredLength || chars.Distinct().Count() < options.RequiredUniqueChars; i++)
        {
            string rcs = randomChars[rand.Next(0, randomChars.Length)];
            chars.Insert(rand.Next(0, chars.Count),
                rcs[rand.Next(0, rcs.Length)]);
        }

        //Вывожу результат
        return new(chars.ToArray());
    }

    /// <summary>
    /// Проверяет <paramref name="email"/> на соответствие критерию электронной почты
    /// Взято с: https://stackoverflow.com/a/1374644/16469671
    /// </summary>
    /// <param name="email">Проверяемая строка</param>
    /// <returns>Является ли <paramref name="email"/> адресом электронной почты</returns>
    public static bool IsValidEmail (string email)
    {
        string trimmedEmail = email.Trim();

        if (trimmedEmail.EndsWith(".", StringComparison.Ordinal))
            return false;

        try
        {
            MailAddress addr = new(email);
            return addr.Address == trimmedEmail;
        }
        catch
        {
            return false;
        }
    }

    /// <summary>
    /// Проверяет текст <paramref name="text"/> на совпадение регулярному выражению по шаблону <paramref name="pattern"/> с опциями <paramref name="options"/> (см. <see cref="RegexOptions"/>)
    /// </summary>
    /// <param name="text">Текст на проверку</param>
    /// <param name="pattern">Шаблон</param>
    /// <param name="options">Параметры проверки в формате <see cref="RegexOptions"/> (можно игнорировать, по умолчанию: <see cref="RegexOptions.IgnoreCase"/>)</param>
    /// <returns>Есть ли совпадения в тексте</returns>
    public static bool IsMatchRegExp (string text, string pattern, RegexOptions? options = null)
    {
        // Задаю настройки проверки регулярных выражений
        RegexOptions regexOptions = options ?? RegexOptions.IgnoreCase;

        // Возвращаю результат
        return Regex.IsMatch(text, pattern, regexOptions);
    }
}