﻿namespace anbs_cp.Classes;

/// <summary>
///     Класс проверки временного интервала
/// </summary>
public static class TimestampValidator
{
    /// <summary>
    ///     Проверка попадания в заданный интервал (в мс)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="checkedStamp">Проверяемый временной интервал</param>
    /// <param name="deltaMs">Временная дельта в миллисекундах</param>
    /// <returns>
    ///     Попал ли <paramref name="checkedStamp" /> в промежуток <paramref name="timestamp" /> +
    ///     <paramref name="deltaMs" />
    /// </returns>
    public static bool Validate(long timestamp, long checkedStamp, ulong deltaMs) =>
        new TimeSpan(timestamp) + TimeSpan.FromMilliseconds(deltaMs) > new TimeSpan(checkedStamp);

    /// <summary>
    ///     Проверка попадания текущего времени в заданный интервал (в мс)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="deltams">Временная дельта в миллисекундах</param>
    /// <returns>Попадает ли текущее время в промежуток <paramref name="timestamp" /> + <paramref name="deltams" /></returns>
    public static bool ValidateNow(long timestamp, ulong deltams) =>
        Validate(timestamp, DateTime.UtcNow.Ticks, deltams);

    /// <summary>
    ///     Проверка временного интервала (в сек)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="checkedstamp">Проверяемый временной интервал</param>
    /// <param name="deltasec">Временная дельта в секундах</param>
    /// <returns>Попал ли <see cref="checkedstamp" /> в промежуток</returns>
    public static bool ValidateFromSec(long timestamp, long checkedstamp, ulong deltasec) =>
        Validate(timestamp, checkedstamp, checked(deltasec * 1000UL));

    /// <summary>
    ///     Проверка попадания текущего времени в заданный интервал (в сек)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="deltasec">Временная дельта в секундах</param>
    /// <returns>Попадает ли текущее время в промежуток <paramref name="timestamp" /> + <paramref name="deltasec" /></returns>
    public static bool ValidateFromSecNow(long timestamp, ulong deltasec) =>
        ValidateFromSec(timestamp, DateTime.UtcNow.Ticks, deltasec);

    /// <summary>
    ///     Проверка временного интервала (в мин)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="checkedstamp">Проверяемый временной интервал</param>
    /// <param name="deltamin">Временная дельта в минутах</param>
    /// <returns>
    ///     Попал ли <see cref="checkedstamp" /> в промежуток <paramref name="timestamp" /> + <paramref name="deltamin" />
    /// </returns>
    public static bool ValidateFromMin(long timestamp, long checkedstamp, ulong deltamin) =>
        Validate(timestamp, checkedstamp, checked(deltamin * 60000UL));

    /// <summary>
    ///     Проверка попадания текущего времени в заданный интервал (в мин)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="deltamin">Временная дельта в минутах</param>
    /// <returns>Попадает ли текущее время в промежуток <paramref name="timestamp" /> + <paramref name="deltamin" /></returns>
    public static bool ValidateFromMinNow(long timestamp, ulong deltamin) =>
        ValidateFromMin(timestamp, DateTime.UtcNow.Ticks, deltamin);

    /// <summary>
    ///     Проверка временного интервала (в час)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="checkedStamp">Проверяемый временной интервал</param>
    /// <param name="deltaHour">Временная дельта в часах</param>
    /// <returns>
    ///     Попал ли <see cref="checkedStamp" /> в промежуток <paramref name="timestamp" /> + <paramref name="deltaHour" />
    /// </returns>
    public static bool ValidateFromHour(long timestamp, long checkedStamp, ulong deltaHour) =>
        Validate(timestamp, checkedStamp, checked(deltaHour * 3600000UL));

    /// <summary>
    ///     Проверка попадания текущего времени в заданный интервал (в час)
    /// </summary>
    /// <param name="timestamp">Временной интервал</param>
    /// <param name="deltahr">Временная дельта в часах</param>
    /// <returns>Попадает ли текущее время в промежуток <paramref name="timestamp" /> + <paramref name="deltahr" /></returns>
    public static bool ValidateFromHourNow(long timestamp, ulong deltahr) =>
        ValidateFromHour(timestamp, DateTime.UtcNow.Ticks, deltahr);
}