﻿using anbs_cp.Database.Interfaces;

using Dapper;

using MySqlConnector;

namespace anbs_cp.Database.Classes;

public class MySqlEngine: IDbEngine
{
    /// <summary>
    /// Конструктор
    /// </summary>
    /// <param name="connectionString">Строка подключения базы данных</param>
    public MySqlEngine (string connectionString) => ConnectionString = connectionString;

    /// <summary>
    /// Строка подключения базы данных
    /// </summary>
    public string ConnectionString { get; set; }

    #region Базовые операции
    /// <summary>
    /// Выполняем команду
    /// </summary>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Количество затронутых строк</returns>
    public async Task<int> ExecuteAsync (string sql, object? values = null)
    {
        // Подключаемся к БД
        await using MySqlConnection connection = new(ConnectionString);

        // Открываем соединение
        await connection.OpenAsync();

        // Выполняем команду и выводим результат
        return await connection.ExecuteAsync(sql, values);
    }

    /// <summary>
    /// Выполняем команду
    /// </summary>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Количество затронутых строк</returns>
    public int Execute (string sql, object? values = null) => ExecuteAsync(sql, values).GetAwaiter().GetResult();

    /// <summary>
    /// Запрос
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    public async Task<IEnumerable<T>> QueryAsync<T> (string sql, object? values = null)
    {
        // Подключаемся к БД
        await using MySqlConnection connection = new(ConnectionString);

        // Открываем соединение
        await connection.OpenAsync();

        // Выполняем запрос и выводим результат
        return await connection.QueryAsync<T>(sql, values);
    }

    /// <summary>
    /// Запрос
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    public IEnumerable<T> Query<T> (string sql, object? values = null) => QueryAsync<T>(sql, values).GetAwaiter().GetResult();

    /// <summary>
    /// Запрос строки
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    public async Task<T?> QueryRowAsync<T> (string sql, object? values = null)
    {
        // Подключаемся к БД
        await using MySqlConnection connection = new(ConnectionString);

        // Открываем соединение
        await connection.OpenAsync();

        // Выполняем запрос и выводим результат
        return await connection.QuerySingleOrDefaultAsync<T>(sql, values);
    }

    /// <summary>
    /// Запрос строки
    /// </summary>
    /// <typeparam name="T">Тип передаваемого значения</typeparam>
    /// <param name="sql">Запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Возвращает массив типа <see cref="T"/> или значение по умолчанию</returns>
    public T? QueryRow<T> (string sql, object? values = null) => QueryRowAsync<T>(sql, values).GetAwaiter().GetResult();

    #endregion


    #region Получение данных
    /// <summary>
    /// Получает массив данных (SELECT * FROM...)
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Массив результата запроса</returns>
    public async Task<IEnumerable<T>> GetResultsAsync<T> (string sql, object? values = null) => await QueryAsync<T>(sql, values);

    /// <summary>
    /// Получает массив данных (SELECT * FROM...)
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Массив результата запроса</returns>
    public IEnumerable<T> GetResults<T> (string sql, object? values = null) => GetResultsAsync<T>(sql, values).GetAwaiter().GetResult();

    /// <summary>
    /// Получает строку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Строки данных или null</returns>
    public async Task<T?> GetRowAsync<T> (string sql, object? values = null)
    {
        // Подключаемся к БД
        await using MySqlConnection connection = new(ConnectionString);

        // Открываем соединение
        await connection.OpenAsync();

        // Выполняем запрос и выводим результат
        return await connection.QuerySingleOrDefaultAsync<T>(sql, values);
    }

    /// <summary>
    /// Получает строку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Строки данных или null</returns>
    public T? GetRow<T> (string sql, object? values = null) => GetRowAsync<T>(sql, values).GetAwaiter().GetResult();

    /// <summary>
    /// Получает колонку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Колонка данных или null</returns>
    public async Task<IEnumerable<T>> GetColAsync<T> (string sql, object? values = null)
        where T : IComparable, IConvertible, IEquatable<T> => await QueryAsync<T>(sql, values);

    /// <summary>
    /// Получает колонку в массиве данных
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Колонка данных или null</returns>
    public IEnumerable<T> GetCol<T> (string sql, object? values = null) where T : IComparable, IConvertible, IEquatable<T> =>
        GetColAsync<T>(sql, values).GetAwaiter().GetResult();

    /// <summary>
    /// Получение значение единичного поля
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Поле или null</returns>
    public async Task<T?> GetVarAsync<T> (string sql, object? values = null) where T : IComparable, IConvertible, IEquatable<T>
    {
        // Подключаемся к БД
        await using MySqlConnection connection = new(ConnectionString);

        // Открываем соединение
        await connection.OpenAsync();

        // Выполняем запрос и выводим результат
        return await connection.QuerySingleOrDefaultAsync<T>(sql, values);
    }

    /// <summary>
    /// Получение значение единичного поля
    /// </summary>
    /// <typeparam name="T">Тип получаемых данных</typeparam>
    /// <param name="sql">SQL-запрос</param>
    /// <param name="values">Данные запроса</param>
    /// <returns>Поле или null</returns>
    public T? GetVar<T> (string sql, object? values = null) where T : IComparable, IConvertible, IEquatable<T> =>
        GetVarAsync<T>(sql, values).GetAwaiter().GetResult();

    #endregion

    #region CRUD данные

    /// <summary>
    /// Вставляет данные в таблицу
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <returns>Результат выполнения</returns>
    public async Task<bool> InsertAsync<T> (T data, string tableName) where T : class
    {
        // Получение список имён свойств в data
        List<string> propertyNamesList = (from dataProperty in data.GetType().GetProperties() select dataProperty.Name).ToList();

        // Получаем список имён в data, обрамленные @
        List<string> propertyValuesList = propertyNamesList.Select(static propertyName => $"@{propertyName}").ToList();

        //Получаем строку имён свойств
        string propertyNames = string.Join(", ", propertyNamesList.ToArray());

        // Получаем строку имён свойств, обрамленных @
        string propertyValues = string.Join(", ", propertyValuesList.ToArray());

        // Создаю соединение
        await using MySqlConnection connection = new(ConnectionString);

        // Создаю запрос
        string sql = $"""
                       INSERT
                       INTO {tableName} ({propertyNames})
                       VALUES ({propertyValues})
                       """;

        // Выполняю запрос
        return await connection.ExecuteAsync(sql, data) > 0;
    }

    /// <summary>
    /// Вставляет данные в таблицу
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <returns>Результат выполнения</returns>
    public bool Insert<T> (T data, string tableName) where T : class => InsertAsync(data, tableName).GetAwaiter().GetResult();

    /// <summary>
    /// Обновляет строку в таблице
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public async Task<bool> UpdateAsync<T> (T data, string tableName, string whereConditionColumn) where T : class
    {
        // Получение список имён свойств в data
        List<string> propertyNamesList = (from dataProperty in data.GetType().GetProperties()
                                          where dataProperty.Name != whereConditionColumn
                                          select dataProperty.Name).ToList();

        // Получаем список имён в data, обрамленные @
        List<string> propertyKeyValuesList = propertyNamesList.Select(static propertyName => $"{propertyName}=@{propertyName}").ToList();

        // Получаем строку имён свойств, обрамленных @
        string properties = string.Join(", ", propertyKeyValuesList.ToArray());

        // Создаю соединение
        await using MySqlConnection connection = new(ConnectionString);

        // Создаю запрос
        string sql = $"""
                       UPDATE {tableName}
                       SET
                       {properties}
                       WHERE
                       {whereConditionColumn}=@{whereConditionColumn}
                       """;

        // Выполняю запрос
        return await connection.ExecuteAsync(sql, data) > 0;
    }

    /// <summary>
    /// Обновляет строку в таблице
    /// </summary>
    /// <typeparam name="T">Класс данных</typeparam>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public bool Update<T> (T data, string tableName, string whereConditionColumn) where T : class =>
        UpdateAsync(data, tableName, whereConditionColumn).GetAwaiter().GetResult();

    /// <summary>
    /// Удаляет строки
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public async Task<bool> DeleteAsync<T> (T data, string tableName, string whereConditionColumn) where T : class
    {
        // Создаю соединение
        await using MySqlConnection connection = new(ConnectionString);

        // Создаю запрос
        string sql = $"""
                       DELETE FROM
                       {tableName}
                       WHERE
                       {whereConditionColumn}=@{whereConditionColumn}
                       """;

        // Выполняю запрос
        return await connection.ExecuteAsync(sql, data) > 0;
    }

    /// <summary>
    /// Удаляет строки
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public bool Delete<T> (T data, string tableName, string whereConditionColumn) where T : class =>
        DeleteAsync(data, tableName, whereConditionColumn).GetAwaiter().GetResult();

    /// <summary>
    /// Удаляет строку
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public async Task<bool> DeleteRowAsync<T> (T data, string tableName, string whereConditionColumn) where T : class
    {
        // Создаю соединение
        await using MySqlConnection connection = new(ConnectionString);

        // Создаю запрос
        string sql = $"""
                       DELETE FROM
                       {tableName}
                       WHERE
                       {whereConditionColumn}=@{whereConditionColumn}
                       LIMIT 1
                       """;

        // Выполняю запрос
        return await connection.ExecuteAsync(sql, data) > 0;
    }

    /// <summary>
    /// Удаляет строку
    /// </summary>
    /// <param name="data">Данные</param>
    /// <param name="tableName">Имя таблицы</param>
    /// <param name="whereConditionColumn">Условие поиска строки. ВНИМАНИЕ! Должно быть одним из указанных свойств типа класса data</param>
    /// <returns>Результат выполнения</returns>
    public bool DeleteRow<T> (T data, string tableName, string whereConditionColumn) where T : class =>
        DeleteRowAsync(data, tableName, whereConditionColumn).GetAwaiter().GetResult();
    #endregion
}